/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * UIFactory.cpp
 *
 *  Created on: Jan 5, 2010
 *      Author: crueger
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include <utility>
#include "CodePatterns/Singleton_impl.hpp"
#include "UIElements/UIFactory.hpp"
#include "CodePatterns/Assert.hpp"
#include "CodePatterns/MemDebug.hpp"

using namespace std;

std::map<std::string,boost::shared_ptr<UIFactory::factoryDescription> > UIFactory::factories;

bool UIFactory::doneInitializing = false;

UIFactory::UIFactory()
{}

UIFactory::~UIFactory()
{
  // clear out factories map to allow boost::shared_ptr to do their work (i.e. to release mem)
  // this is necessary as factories is a static object
  factories.clear();
}

void UIFactory::makeUserInterface(std::string type) {
  ASSERT(factories.count(type),"Selected factory was not registered before creation.");
  // call the factory factory
  setInstance(factories[type]->makeFactory());
}

void UIFactory::registerFactory(factoryDescription *factoryDesc) {
  ASSERT(!factories.count(factoryDesc->name),"Trying to re-register an already registered factory.");
  factories.insert(make_pair(factoryDesc->name,
                             boost::shared_ptr<factoryDescription>(factoryDesc)));
}

CONSTRUCT_SINGLETON(UIFactory)

UIFactory::factoryDescription::factoryDescription(std::string _name) :
  name(_name)
{}

UIFactory::factoryDescription::~factoryDescription()
{}
